function  [S,dccS] = inverse_time(X,A,lambda,op,options)

% INVERSE_TIME source separation in the time domain
%       [S,dccS] = inverse_time(X,A,lambda,options)
%
% Input:
%    X : matrix MxTx of M mixtures of length Tx
%    A : filters MxNxTa
%
% Output:
%    S : maxtrix NxTs of N source signals of length Ts
%    dccS : stft coefficients of S
%
% Note: we must have Ts = Tx - Ta + 1 to deal with the finite convolution

[~,N,K] = size(A);
TT = size(X,2);
T = TT - K + 1;


if ~isfield(options, 'penalty')
    options.penalty = 'l12';
end

if ~isfield(options, 'maxit')
         options.maxit = 20000;
end


if ~isfield(options, 'lipschitz')
    %% Choice of C: square of the upper bound norm of the operator.
    if K==1 % Instantaneous mixture
        C = norm(A,'fro')^2 + eps*10^8;
    else
        options.maxitl=500;
        C = 1.01*power_iteration(A,T,options);
    end
    options.lipschitz = C;
end



%% Calculation of the adjoint operator applied to X
if K==1
    AtX = A'*X;
else
    At = retourne_mat(A);
    AtX = creation_mel_conv(At,X);
    AtX = AtX(:,K:T+K-1);
end

tr = op.analysis(AtX(1,:));
[Mtr,Ntr] = size(tr);
AtdccX = zeros(Mtr,Ntr,N);


if ~isfield(options, 'init')
    options.init = zeros(Mtr,Ntr,N);
end
dccS0 = options.init;

for n=1:N
    tr = op.analysis(AtX(n,:));
    AtdccX(:,:,n) = tr;
end
%% End Calculation

dccS = dccS0;




%% Some initializations

y = dccS0;
iC = 1/options.lipschitz;

%% Main loop: fista iterations
for it=1:options.maxit
    old = dccS;
    
    dccS = y - iC*compute_grad_time_aux(y);
    dccS = prox(dccS,iC*lambda,options.penalty);
    
    y = dccS + (it-1)/(it+4) * (dccS-old);
    
    if norm(old(:)-dccS(:))/(norm(dccS(:))+eps) < 1e-4
        break;
    end
    
    %    fprintf('it = %d  %f %0.12f\n',it,norm(dccS(:)-old(:))/norm(dccS(:)),norm(dccS(:)-old(:)));
end



%% Back to the time domain

S = zeros(N,T);
for k=1:N
    s = op.synthesis(dccS(:,:,k));
    S(k,:) = s(1:T)';
end



    function [grad]=compute_grad_time_aux(dcc)
        
        Sg = zeros(N,TT);
        AtAdcc = zeros(Mtr,Ntr,N);
        
        if K==1 % Instantaneous mixture
            dccr = permute(dcc,[3,1,2]);
            dccr = reshape(dccr,N,T);
            AtAdcc = A'*(A*dccr);
            AtAdcc = reshape(AtAdcc,[N,Mtr,Ntr]);
            AtAdcc = permute(AtAdcc,[2,3,1]);
        else
            for ng=1:N
                Sg(ng,:) = op.synthesis(dcc(:,:,ng));
            end
            AS =  creation_mel_conv(A,Sg);
            AtAS = creation_mel_conv(At,AS);
            AtAS = AtAS(:,K:T+K-1);
            for ng=1:N
                AtAdcc(:,:,ng) = op.analysis(AtAS(ng,:));
            end
        end
        
        grad = AtAdcc-AtdccX;
    end

end

